;---------------------------------------------------------------------
; Graphical User Interface             Copyright (C) 2009 Miguel Dutra
;---------------------------------------------------------------------

.386                                                                    ; Force 32-bit code
.model flat, stdcall                                                    ; Memory model & calling convention
option casemap :none                                                    ; Case sensitive

include windows.inc                                                     ; Windows typedefs
include kernel32.inc                                                    ; GetCommandLine, ExitProcess
include user32.inc                                                      ; DialogBoxParam, wsprintf
include comctl32.inc                                                    ; InitCommonControls
include comdlg32.inc                                                    ; GetOpenFileName
include advapi32.inc                                                    ; RegOpenKeyEx, RegQueryValueEx, RegCloseKey
include shell32.inc                                                     ; SHBrowseForFolder
include masm32.inc                                                      ; MemCopy, StdOut
include macros.asm                                                      ; fn, fnc

includelib kernel32.lib
includelib user32.lib
includelib comctl32.lib
includelib comdlg32.lib
includelib advapi32.lib
includelib shell32.lib
includelib masm32.lib

include ..\macros\macros.asm
include ..\VDI\VDI.asm
include ..\JV1\JV1.asm
include ..\JV3\JV3.asm
include ..\DMK\DMK.asm
include ..\OSI\OSI.asm
include ..\TRSDOS\TRSDOS.asm
include ..\DOSPLUS\DOSPLUS.asm
include ..\MICRODOS\MICRODOS.asm
include ..\MULTIDOS\MULTIDOS.asm
include ..\NEWDOS80\NEWDOS80.asm
include ..\RAPIDOS\RAPIDOS.asm
include ..\TRSDOS1\TRSDOS1.asm
include ..\TRSDOS2\TRSDOS2.asm
include ..\TRSDOS27\TRSDOS27.asm

include resource.inc

;---------------------------------------------------------------------
; Function Prototypes
;---------------------------------------------------------------------

GUI_ColumnSort              proto   :LPARAM, :LPARAM, :LPARAM

GUI_GetClass                proto
GUI_SetClass                proto
GUI_ResetClass              proto

GUI_List                    proto   :HWND
GUI_GetDateStr              proto   :DWORD, :DWORD, :DWORD
GUI_GetAttrStr              proto   :DWORD, :DWORD, :DWORD
GUI_AddListItem             proto   :HWND, :DWORD, :DWORD, :DWORD, :DWORD, :DWORD, :DWORD, :DWORD

GUI_LoadOSI                 proto   :DWORD, :DWORD, :DWORD
GUI_LoadVDI                 proto   :DWORD, :HANDLE, :DWORD

GUI_OpenFile                proto   :HWND, :DWORD, :DWORD, :DWORD
GUI_CloseFile               proto   :HWND

GUI_OpenFileFromResDlg      proto   :HWND
GUI_OpenFileFromSysDlg      proto   :HWND
GUI_OpenFileFromCmdLine     proto   :HWND

GUI_LoadSettings            proto
GUI_SaveSettings            proto

;---------------------------------------------------------------------
; Global Variables
;---------------------------------------------------------------------

.data

argv                DWORD       16  dup (?)                             ; Argument values array
argc                BYTE        ?                                       ; Argument count

GUI_hFile           HANDLE      INVALID_HANDLE_VALUE                    ; Currently open file handle
GUI_FileName        BYTE        MAX_PATH dup (0)                        ; Currently open file name

GUI_VDI             CVDI        <>                                      ; VDI object
GUI_OSI             COSI        <>                                      ; OSI object

GUI_HKCU_Key        BYTE        "Software\Miguel Dutra\VDISK", 0        ; User preferences
GUI_HKCR_Class      BYTE        "Miguel Dutra.VDISK.1", 0               ; Program class
GUI_HKCR_Descr      BYTE        "TRS-80 Floppy Image", 0                ; Description of files associated with the class

GUI_Main_Sort_Sel   DWORD       0                                       ; Currently selected column
GUI_Main_Sort_Order BYTE        0                                       ; Ascending or descending order

;---------------------------------------------------------------------
; Settings
;---------------------------------------------------------------------

GUI_CONFIG  struct
    Associate       DWORD       ?                                       ; File association status
    AutoDetect      DWORD       ?                                       ; Auto-detect disk format
    HideSystem      DWORD       ?                                       ; Hide System files
    HideInvisible   DWORD       ?                                       ; Hide Invisible files
    WindowWidth     DWORD       ?                                       ; Window width
    WindowHeight    DWORD       ?                                       ; Window height
GUI_CONFIG  ends

GUI_Config          GUI_CONFIG      <0, 1, 0, 0, 500, 400>              ; Initial configuration values

GUI_SETTING struct
    pName           DWORD       ?                                       ; Pointer to the setting's Name
    pValue          DWORD       ?                                       ; Pointer to the setting's Value
    pMII            DWORD       ?                                       ; Pointer to the setting's Menu Item Info
GUI_SETTING ends

GUI_Setting_1       BYTE        "Associate", 0
GUI_Setting_2       BYTE        "AutoDetect", 0
GUI_Setting_3       BYTE        "HideSystem", 0
GUI_Setting_4       BYTE        "HideInvisible", 0
GUI_Setting_5       BYTE        "WindowWidth", 0
GUI_Setting_6       BYTE        "WindowHeight", 0

GUI_MMI             MENUITEMINFO    <sizeof MENUITEMINFO, MIIM_STATE, 0, MFS_UNCHECKED, IDM_OPTIONS_ASSOCIATE,   NULL, NULL, NULL, 0, 0, 0>
                    MENUITEMINFO    <sizeof MENUITEMINFO, MIIM_STATE, 0, MFS_UNCHECKED, IDM_OPTIONS_AUTODETECT,  NULL, NULL, NULL, 0, 0, 0>
                    MENUITEMINFO    <sizeof MENUITEMINFO, MIIM_STATE, 0, MFS_UNCHECKED, IDM_OPTIONS_HIDESYS,     NULL, NULL, NULL, 0, 0, 0>
                    MENUITEMINFO    <sizeof MENUITEMINFO, MIIM_STATE, 0, MFS_UNCHECKED, IDM_OPTIONS_HIDEINV,     NULL, NULL, NULL, 0, 0, 0>

GUI_MMI_FA          equ         0                                       ; 0:File Association
GUI_MMI_AD          equ         1                                       ; 1:Auto-Detect
GUI_MMI_SS          equ         2                                       ; 2:Hide System
GUI_MMI_SI          equ         3                                       ; 3:Hide Invisible

GUI_Settings        GUI_SETTING     <offset GUI_Setting_1, offset GUI_Config.Associate,     offset GUI_MMI[MENUITEMINFO * 0]>
                    GUI_SETTING     <offset GUI_Setting_2, offset GUI_Config.AutoDetect,    offset GUI_MMI[MENUITEMINFO * 1]>
                    GUI_SETTING     <offset GUI_Setting_3, offset GUI_Config.HideSystem,    offset GUI_MMI[MENUITEMINFO * 2]>
                    GUI_SETTING     <offset GUI_Setting_4, offset GUI_Config.HideInvisible, offset GUI_MMI[MENUITEMINFO * 3]>
                    GUI_SETTING     <offset GUI_Setting_5, offset GUI_Config.WindowWidth,   NULL>
                    GUI_SETTING     <offset GUI_Setting_6, offset GUI_Config.WindowHeight,  NULL>

GUI_MMI_Count       equ         4                                       ; Number of Menu Items
GUI_Settings_Count  equ         6                                       ; Number of Registry Settings

;---------------------------------------------------------------------
; Column Headers
;---------------------------------------------------------------------

GUI_Col_Name        BYTE        "Name", 0
GUI_Col_Extension   BYTE        "Extension", 0
GUI_Col_Size        BYTE        "Size", 0
GUI_Col_Date        BYTE        "Date", 0
GUI_Col_Attributes  BYTE        "Attributes", 0

GUI_LVC_Name        LV_COLUMN   <LVCF_FMT+LVCF_TEXT+LVCF_WIDTH, LVCFMT_LEFT,   80, offset GUI_Col_Name,        , >
GUI_LVC_Extension   LV_COLUMN   <LVCF_FMT+LVCF_TEXT+LVCF_WIDTH, LVCFMT_LEFT,   80, offset GUI_Col_Extension,   , >
GUI_LVC_Size        LV_COLUMN   <LVCF_FMT+LVCF_TEXT+LVCF_WIDTH, LVCFMT_RIGHT,  80, offset GUI_Col_Size,        , >
GUI_LVC_Date        LV_COLUMN   <LVCF_FMT+LVCF_TEXT+LVCF_WIDTH, LVCFMT_CENTER, 80, offset GUI_Col_Date,        , >
GUI_LVC_Attributes  LV_COLUMN   <LVCF_FMT+LVCF_TEXT+LVCF_WIDTH, LVCFMT_CENTER, 80, offset GUI_Col_Attributes,  , >

;---------------------------------------------------------------------
; Open Dialog Filter
;---------------------------------------------------------------------

GUI_Filter_1        BYTE        "Virtual Floppy Disks (*.dsk, *.dmk)", 0
                    BYTE        "*.dsk;*.dmk", 0
GUI_Filter_2        BYTE        "All Files (*.*)", 0
                    BYTE        "*.*", 0
                    BYTE        0

;---------------------------------------------------------------------
; Interface Dialog Items
;---------------------------------------------------------------------

GUI_SEL_Auto        BYTE        "AUTO", 0
GUI_SEL_DMK         BYTE        "DMK", 0
GUI_SEL_JV1         BYTE        "JV1", 0
GUI_SEL_JV3         BYTE        "JV3", 0
GUI_SEL_CPM         BYTE        "CP/M Compatible", 0
GUI_SEL_DBLDOS      BYTE        "DBLDOS Compatible", 0
GUI_SEL_DOSPLUS     BYTE        "DOSPLUS Compatible", 0
GUI_SEL_MICRODOS    BYTE        "MICRODOS Compatible", 0
GUI_SEL_MULTIDOS    BYTE        "MULTIDOS Compatible", 0
GUI_SEL_NEWDOS80    BYTE        "NEWDOS/80 Compatible", 0
GUI_SEL_OS80        BYTE        "OS/80 III Compatible", 0
GUI_SEL_RAPIDOS     BYTE        "RAPIDOS Compatible", 0
GUI_SEL_TRSDOS1     BYTE        "TRSDOS 1.x Compatible", 0
GUI_SEL_TRSDOS2     BYTE        "TRSDOS 2.x Compatible", 0
GUI_SEL_TRSDOS27    BYTE        "TRSDOS 2.7 Compatible", 0
GUI_SEL_TRSDOS6     BYTE        "TRSDOS 6.x Compatible", 0

GUI_SEL_VDI  struct
    pName           DWORD       ?
    pVTBL           DWORD       ?
GUI_SEL_VDI  ends

GUI_SEL_OSI  struct
    pName           DWORD       ?
    pVTBL           DWORD       ?
GUI_SEL_OSI  ends

GUI_SEL_VDI_Array   GUI_SEL_VDI  <offset GUI_SEL_Auto,        -1>
                    GUI_SEL_VDI  <offset GUI_SEL_DMK,         offset DMK>
                    GUI_SEL_VDI  <offset GUI_SEL_JV1,         offset JV1>
                    GUI_SEL_VDI  <offset GUI_SEL_JV3,         offset JV3>

GUI_SEL_VDI_Count   equ         4

GUI_SEL_OSI_Array   GUI_SEL_OSI  <offset GUI_SEL_Auto,        -1>
;                    GUI_SEL_OSI  <offset GUI_SEL_CPM,         -1>
;                    GUI_SEL_OSI  <offset GUI_SEL_DBLDOS,      -1>
                    GUI_SEL_OSI  <offset GUI_SEL_DOSPLUS,     offset DOSPLUS>
                    GUI_SEL_OSI  <offset GUI_SEL_MICRODOS,    offset MICRODOS>
                    GUI_SEL_OSI  <offset GUI_SEL_MULTIDOS,    offset MULTIDOS>
                    GUI_SEL_OSI  <offset GUI_SEL_NEWDOS80,    offset NEWDOS80>
;                    GUI_SEL_OSI  <offset GUI_SEL_OS80,        -1>
                    GUI_SEL_OSI  <offset GUI_SEL_RAPIDOS,     offset RAPIDOS>
                    GUI_SEL_OSI  <offset GUI_SEL_TRSDOS1,     offset TRSDOS1>
                    GUI_SEL_OSI  <offset GUI_SEL_TRSDOS2,     offset TRSDOS2>
                    GUI_SEL_OSI  <offset GUI_SEL_TRSDOS27,    offset TRSDOS27>
                    GUI_SEL_OSI  <offset GUI_SEL_TRSDOS6,     offset TRSDOS>

GUI_SEL_OSI_Count   equ         10

.code

include GUI_Library.asm
include GUI_Menu.asm
include GUI_OpenDialog.asm
include GUI_ExportDialog.asm
include GUI_PropertiesDialog.asm
include GUI_ViewerDialog.asm
include GUI_AboutDialog.asm
include GUI_MainDialog.asm

;---------------------------------------------------------------------
; EntryPoint
;---------------------------------------------------------------------

EntryPoint: fn      InitCommonControls

            fn      GetModuleHandle, NULL

            fn      DialogBoxParam, eax, IDD_MAIN, NULL, offset GUI_Main, NULL

            fn      ExitProcess, 0

;---------------------------------------------------------------------
; GUI_ColumnSort                      Directory list sorting procedure
;---------------------------------------------------------------------

GUI_ColumnSort      proc    lParam1:LPARAM, lParam2:LPARAM, lParamSort:LPARAM

            local   Str1[32]:BYTE
            local   Str2[32]:BYTE

    ; --- Sort by column "Name" ---
                
            cmp     lParamSort, 0
            jne     @F

            mcall   OSI_VTBL, GetFileName, addr GUI_OSI, lParam1, addr Str1, sizeof Str1
            mcall   OSI_VTBL, GetFileName, addr GUI_OSI, lParam2, addr Str2, sizeof Str2

            push    esi
            push    edi

            lea     esi, Str1
            lea     edi, Str2
            mov     ecx, 8
            cld
            repe    cmpsb

            pop     edi
            pop     esi
            jmp     Finish

    ; --- Sort by column "Extension" ---
                
@@:         cmp     lParamSort, 1
            jne     @F

            mcall   OSI_VTBL, GetFileExtension, addr GUI_OSI, lParam1, addr Str1, sizeof Str1
            mcall   OSI_VTBL, GetFileExtension, addr GUI_OSI, lParam2, addr Str2, sizeof Str2

            push    esi
            push    edi

            lea     esi, Str1
            lea     edi, Str2
            mov     ecx, 3
            cld
            repe    cmpsb

            pop     edi
            pop     esi
            jmp     Finish

    ; --- Sort by column "Size" ---
                
@@:         cmp     lParamSort, 2
            jne     @F

            mcall   OSI_VTBL, GetFileSize, addr GUI_OSI, lParam1
            push    eax
            mcall   OSI_VTBL, GetFileSize, addr GUI_OSI, lParam2
            pop     ecx

            cmp     eax, ecx
            jmp     Finish

    ; --- Sort by column "Date" ---
                
@@:         cmp     lParamSort, 3
            jne     @F

            mcall   OSI_VTBL, GetFileDate, addr GUI_OSI, lParam1
            push    eax
            mcall   OSI_VTBL, GetFileDate, addr GUI_OSI, lParam2
            pop     ecx

            cmp     eax, ecx
            jmp     Finish

    ; --- Sort by column "Attributes" ---
                
@@:         cmp     lParamSort, 4
            jne     Done

            mcall   OSI_VTBL, GetFileAttributes, addr GUI_OSI, lParam1
            push    eax
            mcall   OSI_VTBL, GetFileAttributes, addr GUI_OSI, lParam2
            pop     ecx

            cmp     eax, ecx
            jmp     Finish

    ; --- Set EAX according to ZR and CY flag states after comparision ---
                
Finish:     setnz   al                                      ; Set AL=0 if ZR, -127 if CY, +127 if NC
            rcr     al, 1
            adc     al, 0

            movsx   eax, al

            cmp     GUI_Main_Sort_Order, 0
            je      Done

            neg     eax                                     ; Invert the sorting order
            
Done:       ret
            
GUI_ColumnSort      endp

;---------------------------------------------------------------------
; GUI_GetClass                          Return file association status
;---------------------------------------------------------------------
; No input values
;---------------------------------------------------------------------
; EAX:      File association status (0:FALSE, 1:TRUE)
;---------------------------------------------------------------------

GUI_GetClass        proc

            local   hKey:PHKEY
            local   Buffer[256]:BYTE
            local   ValueType:DWORD
            local   Len:DWORD
            local   DMK_Flag:BYTE
            local   DSK_Flag:BYTE

    ; --- Get the status of DSK files ---

            mov     Len, sizeof Buffer

            fn      RegOpenKeyEx, HKEY_CLASSES_ROOT, ".DSK", 0, KEY_READ, addr hKey

            fn      RegQueryValueEx, hKey, NULL, NULL, addr ValueType, addr Buffer, addr Len

            fn      RegCloseKey, hKey

            push    esi
            push    edi

            lea     esi, Buffer
            mov     edi, offset GUI_HKCR_Class
            mov     ecx, sizeof GUI_HKCR_Class
            cld
            repe    cmpsb

            pop     edi
            pop     esi

            sete    DSK_Flag

    ; --- Get the status of DMK files ---

            fn      RegOpenKeyEx, HKEY_CLASSES_ROOT, ".DMK", 0, KEY_READ, addr hKey

            fn      RegQueryValueEx, hKey, NULL, NULL, addr ValueType, addr Buffer, addr Len

            fn      RegCloseKey, hKey

            push    esi
            push    edi

            lea     esi, Buffer
            mov     edi, offset GUI_HKCR_Class
            mov     ecx, sizeof GUI_HKCR_Class
            cld
            repe    cmpsb

            pop     edi
            pop     esi

            sete    DMK_Flag

    ; --- Set return register ---

            xor     eax, eax

            mov     al, DSK_Flag
            and     al, DMK_Flag

            ret

GUI_GetClass        endp

;---------------------------------------------------------------------
; GUI_SetClass                             Establish file associations
;---------------------------------------------------------------------
; No input values
;---------------------------------------------------------------------
; EAX:      Error code
;---------------------------------------------------------------------

GUI_SetClass        proc

            local   hKey:PHKEY
            local   File[MAX_PATH]:BYTE
            local   String[256]:BYTE
            local   KeyStatus:DWORD

    ; --- Get current process' file and path ---

            fn      GetModuleFileName, NULL, addr File, sizeof File

            or      eax, eax
            jnz     @F

            fn      GetLastError
            jmp     Done

    ; --- Register program class ---

@@:         fn      RegCreateKeyEx, HKEY_CLASSES_ROOT, offset GUI_HKCR_Class, 0, "?", 0, KEY_WRITE, NULL, addr hKey, addr KeyStatus

            or      eax, eax
            jnz     Done

            fn      RegSetValueEx, hKey, NULL, 0, REG_SZ, offset GUI_HKCR_Descr, sizeof GUI_HKCR_Descr

            or      eax, eax
            jnz     Done

            fn      RegCloseKey, hKey

    ; --- Set path to application icon ---

            fn      wsprintf, addr String, "%s\DefaultIcon", offset GUI_HKCR_Class

            fn      RegCreateKeyEx, HKEY_CLASSES_ROOT, addr String, 0, "?", 0, KEY_WRITE, NULL, addr hKey, addr KeyStatus

            or      eax, eax
            jnz     Done

            fn      wsprintf, addr String, "%s, %d", addr File, IDI_ICON

            fn      RegSetValueEx, hKey, NULL, 0, REG_SZ, addr String, sizeof String

            or      eax, eax
            jnz     Done

            fn      RegCloseKey, hKey

    ; --- Set Open shell command ---

            fn      wsprintf, addr String, "%s\Shell\Open\command", offset GUI_HKCR_Class

            fn      RegCreateKeyEx, HKEY_CLASSES_ROOT, addr String, 0, "?", 0, KEY_WRITE, NULL, addr hKey, addr KeyStatus

            or      eax, eax
            jnz     Done

            fn      wsprintf, addr String, "%s ""%%1""", addr File

            fn      RegSetValueEx, hKey, NULL, 0, REG_SZ, addr String, sizeof String

            or      eax, eax
            jnz     Done

            fn      RegCloseKey, hKey

    ; --- Establish association with DSK files ---

            fn      RegCreateKeyEx, HKEY_CLASSES_ROOT, ".DSK", 0, "?", 0, KEY_WRITE, NULL, addr hKey, addr KeyStatus

            or      eax, eax
            jnz     Done

            fn      RegSetValueEx, hKey, NULL, 0, REG_SZ, offset GUI_HKCR_Class, sizeof GUI_HKCR_Class

            or      eax, eax
            jnz     Done

            fn      RegCloseKey, hKey

    ; --- Establish association with DMK files ---

            fn      RegCreateKeyEx, HKEY_CLASSES_ROOT, ".DMK", 0, "?", 0, KEY_WRITE, NULL, addr hKey, addr KeyStatus

            or      eax, eax
            jnz     Done

            fn      RegSetValueEx, hKey, NULL, 0, REG_SZ, offset GUI_HKCR_Class, sizeof GUI_HKCR_Class

            or      eax, eax
            jnz     Done

            fn      RegCloseKey, hKey

Done:       ret

GUI_SetClass        endp

;---------------------------------------------------------------------
; GUI_ResetClass                              Remove file associations
;---------------------------------------------------------------------
; No input values
;---------------------------------------------------------------------
; EAX:      Error code
;---------------------------------------------------------------------

GUI_ResetClass      proc

            local   hKey:PHKEY

    ; --- Remove association with DSK files ---

            fn      RegOpenKeyEx, HKEY_CLASSES_ROOT, ".DSK", 0, KEY_WRITE, addr hKey

            or      eax, eax
            jnz     @F

            fn      RegSetValueEx, hKey, NULL, 0, REG_SZ, NULL, 0

            or      eax, eax
            jnz     Done

            fn      RegCloseKey, hKey

    ; --- Remove association with DMK files ---

@@:         fn      RegOpenKeyEx, HKEY_CLASSES_ROOT, ".DMK", 0, KEY_WRITE, addr hKey

            or      eax, eax
            jnz     @F

            fn      RegSetValueEx, hKey, NULL, 0, REG_SZ, NULL, 0

            or      eax, eax
            jnz     Done

            fn      RegCloseKey, hKey

@@:         mov     eax, NO_ERROR

Done:       ret

GUI_ResetClass      endp

;---------------------------------------------------------------------
; GUI_List                                  Load the directory listing
;---------------------------------------------------------------------
; hWnd:     Window handle
;---------------------------------------------------------------------
; No return value
;---------------------------------------------------------------------

GUI_List    proc    hWnd:HWND

            local   hFPDE:DWORD
            local   Index:DWORD
            local   FindFlag:BYTE
            local   FileName[9]:BYTE
            local   FileExt[4]:BYTE
            local   FileSizeStr[9]:BYTE
            local   FileDateStr[11]:BYTE
            local   FileAttrStr[9]:BYTE
            local   FileAttr:DWORD

            mov     Index, 0

    ; --- Clear controls ---
    
            fn      GetDlgItem, hWnd, IDC_LIST

            fn      SendMessage, eax, LVM_DELETEALLITEMS, 0, 0

    ; --- Get next directory entry ---

            mov     FindFlag, OSI_FINDFIRST

Dir:        mcall   OSI_VTBL, FindFiles, addr GUI_OSI, FindFlag

            or      eax, eax                                            ; No more files
            jz      Done

            cmp     eax, -1                                             ; Error
            jne     @F

            mov     eax, ERROR_SEEK
            jmp     Done

@@:         mov     hFPDE, eax

    ; --- Get file information ---

            mcall   OSI_VTBL, GetFileName, addr GUI_OSI, hFPDE, addr FileName, sizeof FileName
            mov     FileName[8], 0

            mcall   OSI_VTBL, GetFileExtension, addr GUI_OSI, hFPDE, addr FileExt, sizeof FileExt
            mov     FileExt[3], 0

            mcall   OSI_VTBL, GetFileSize, addr GUI_OSI, hFPDE

            fn      wsprintf, addr FileSizeStr, "%i", eax

            mcall   OSI_VTBL, GetFileDate, addr GUI_OSI, hFPDE

            fn      GUI_GetDateStr, addr FileDateStr, sizeof FileDateStr, eax

            mcall   OSI_VTBL, GetFileAttributes, addr GUI_OSI, hFPDE
            mov     FileAttr, eax

            fn      GUI_GetAttrStr, addr FileAttrStr, sizeof FileAttrStr, eax

    ; --- Check attribute filters ---

            xor     eax, eax

            test    FileAttr, 000000F0h                                 ; Check the "System" attribute
            jz      @F

            cmp     GUI_Config.HideSystem, 0                            ; Check the "System" filter
            jne     Next

@@:         test    FileAttr, 00000F00h                                 ; Check the "Invisible" attribute
            jz      @F

            cmp     GUI_Config.HideInvisible, 0                         ; Check the "Invisible" filter
            jne     Next

    ; --- Insert file in the list control and advance to the next entry ---

@@:         fn      GUI_AddListItem, hWnd, Index, hFPDE, addr FileName, addr FileExt, addr FileSizeStr, addr FileDateStr, addr FileAttrStr

            inc     Index

Next:       mov     FindFlag, OSI_FINDNEXT
            jmp     Dir

    ; --- Repaint the control ---

Done:       fn      GetDlgItem, hWnd, IDC_LIST

            fn      SendMessage, eax, WM_PAINT, 0, 0

            ret

GUI_List    endp

;---------------------------------------------------------------------
; GUI_GetDateStr                   Return a formatted file date string
;---------------------------------------------------------------------
; pStr:     Pointer to the target buffer
; Len:      Length of the target buffer
; Date:     File date (YYYYMMDD)
;---------------------------------------------------------------------
; EAX:      Error code
;---------------------------------------------------------------------

GUI_GetDateStr      proc    pStr:DWORD, Len:DWORD, Date:DWORD

            cmp     Len, 11
            jae     @F

            mov     eax, ERROR_INSUFFICIENT_BUFFER
            jmp     Done

@@:         mov     eax, Date

            movzx   edx, al                                             ; Day
            push    edx

            movzx   edx, ah                                             ; Month
            push    edx

            shr     eax, 16                                             ; Year
            push    eax

            fn      wsprintf, pStr, "%04i-%02i-%02d"

            add     esp, 3*4                                            ; Re-balance stack

            mov     eax, NO_ERROR

Done:       ret

GUI_GetDateStr      endp

;---------------------------------------------------------------------
; GUI_GetAttrStr              Return a formatted file attribute string
;---------------------------------------------------------------------
; pStr:     Pointer to the target buffer
; Len:      Length of the target buffer
; Attr:     File attributes (0000MISA)
;---------------------------------------------------------------------
; EAX:      Error code
;---------------------------------------------------------------------

GUI_GetAttrStr      proc    pStr:DWORD, Len:DWORD, Attr:DWORD

            cmp     Len, 9
            jae     @F

            mov     eax, ERROR_INSUFFICIENT_BUFFER
            jmp     Done

@@:         fn      MemCopy, "-------0", pStr, 9

            mov     eax, Attr
            mov     edx, pStr

            test    eax, 0000F000h
            jz      @F

            mov     byte ptr [edx + 4], 'M'

@@:         test    eax, 00000F00h
            jz      @F

            mov     byte ptr [edx + 5], 'I'

@@:         test    eax, 000000F0h
            jz      @F

            mov     byte ptr [edx + 6], 'S'

@@:         and     al, 0Fh
            add     [edx + 7], al

            mov     eax, NO_ERROR

Done:       ret

GUI_GetAttrStr      endp

;---------------------------------------------------------------------
; GUI_AddListItem            Insert file info in the list view control
;---------------------------------------------------------------------
; hWnd:         Window handle
; Index:        File index (0-n)
; lParam:       Any value to be assigned to the item
; pName:        Pointer to a null-terminated file name
; pExtension:   Pointer to a null-terminated file extension
; pSize:        Pointer to a null-terminated file size string
; pDate:        Pointer to a null-terminated file date string
; pAttrib:      Pointer to a null-terminated file attributes string
;---------------------------------------------------------------------
; No return value
;---------------------------------------------------------------------

GUI_AddListItem     proc    hWnd:HWND, Index:DWORD, lParam:DWORD, pName:DWORD, pExtension:DWORD, pSize:DWORD, pDate:DWORD, pAttrib:DWORD

            local   ListItem[5]:LV_ITEM
            local   hList:HWND

            mov     ecx, Index
            mov     edx, lParam

            mov     eax, pName

            mov     ListItem[sizeof LV_ITEM * 0].imask, LVIF_TEXT+LVIF_PARAM
            mov     ListItem[sizeof LV_ITEM * 0].iItem, ecx
            mov     ListItem[sizeof LV_ITEM * 0].iSubItem, 0
            mov     ListItem[sizeof LV_ITEM * 0].pszText, eax
            mov     ListItem[sizeof LV_ITEM * 0].lParam, edx

            mov     eax, pExtension

            mov     ListItem[sizeof LV_ITEM * 1].imask, LVIF_TEXT
            mov     ListItem[sizeof LV_ITEM * 1].iItem, ecx
            mov     ListItem[sizeof LV_ITEM * 1].iSubItem, 1
            mov     ListItem[sizeof LV_ITEM * 1].pszText, eax

            mov     eax, pSize

            mov     ListItem[sizeof LV_ITEM * 2].imask, LVIF_TEXT
            mov     ListItem[sizeof LV_ITEM * 2].iItem, ecx
            mov     ListItem[sizeof LV_ITEM * 2].iSubItem, 2
            mov     ListItem[sizeof LV_ITEM * 2].pszText, eax

            mov     eax, pDate

            mov     ListItem[sizeof LV_ITEM * 3].imask, LVIF_TEXT
            mov     ListItem[sizeof LV_ITEM * 3].iItem, ecx
            mov     ListItem[sizeof LV_ITEM * 3].iSubItem, 3
            mov     ListItem[sizeof LV_ITEM * 3].pszText, eax

            mov     eax, pAttrib

            mov     ListItem[sizeof LV_ITEM * 4].imask, LVIF_TEXT
            mov     ListItem[sizeof LV_ITEM * 4].iItem, ecx
            mov     ListItem[sizeof LV_ITEM * 4].iSubItem, 4
            mov     ListItem[sizeof LV_ITEM * 4].pszText, eax

            fn      GetDlgItem, hWnd, IDC_LIST                          ; Get handle to list view control
            mov     hList, eax

            fn      SendMessage, hList, LVM_INSERTITEM, 0, addr ListItem[sizeof LV_ITEM * 0]
            fn      SendMessage, hList, LVM_SETITEM,    1, addr ListItem[sizeof LV_ITEM * 1]
            fn      SendMessage, hList, LVM_SETITEM,    2, addr ListItem[sizeof LV_ITEM * 2]
            fn      SendMessage, hList, LVM_SETITEM,    3, addr ListItem[sizeof LV_ITEM * 3]
            fn      SendMessage, hList, LVM_SETITEM,    4, addr ListItem[sizeof LV_ITEM * 4]

            ret

GUI_AddListItem     endp

;---------------------------------------------------------------------
; GUI_LoadOSI                          Load Operating System Interface
;---------------------------------------------------------------------
; pOSI:         Pointer to OSI object
; pVDI:         Pointer to an initialized VDI object
; Interface:    Selected OSI
;---------------------------------------------------------------------
; EAX:          Error code
;---------------------------------------------------------------------

GUI_LoadOSI         proc    pOSI:DWORD, pVDI:DWORD, Interface:DWORD

    ; --- Load the requested interface ---

            cmp     Interface, -1
            je      Auto

            mov     edx, pOSI
            mov     eax, Interface
            mov     (COSI PTR [edx]).VPTR, eax

            mcall   OSI_VTBL, Load, pOSI, pVDI

            jmp     Done

    ; --- Try the NEWDOS80 interface ---

Auto:       mov     eax, pOSI

            mov     (COSI PTR [eax]).VPTR, offset NEWDOS80

            mcall   OSI_VTBL, Load, pOSI, pVDI

            or      eax, eax
            jz      Done

            mcall   OSI_VTBL, Unload, pOSI

    ; --- Try the MICRODOS interface ---

            mov     eax, pOSI

            mov     (COSI PTR [eax]).VPTR, offset MICRODOS

            mcall   OSI_VTBL, Load, pOSI, pVDI

            or      eax, eax
            jz      Done

            mcall   OSI_VTBL, Unload, pOSI

    ; --- Try the DOSPLUS interface ---

            mov     eax, pOSI

            mov     (COSI PTR [eax]).VPTR, offset DOSPLUS

            mcall   OSI_VTBL, Load, pOSI, pVDI

            or      eax, eax
            jz      Done

            mcall   OSI_VTBL, Unload, pOSI

    ; --- Try the TRSDOS interface ---

            mov     eax, pOSI

            mov     (COSI PTR [eax]).VPTR, offset TRSDOS

            mcall   OSI_VTBL, Load, pOSI, pVDI

            or      eax, eax
            jz      Done

            mcall   OSI_VTBL, Unload, pOSI

    ; --- Try the TRSDOS1 interface ---

            mov     eax, pOSI

            mov     (COSI PTR [eax]).VPTR, offset TRSDOS1

            mcall   OSI_VTBL, Load, pOSI, pVDI

            or      eax, eax
            jz      Done

            mcall   OSI_VTBL, Unload, pOSI

    ; --- Try the TRSDOS2 interface ---

            mov     eax, pOSI

            mov     (COSI PTR [eax]).VPTR, offset TRSDOS2

            mcall   OSI_VTBL, Load, pOSI, pVDI

            or      eax, eax
            jz      Done

            mcall   OSI_VTBL, Unload, pOSI

    ; --- Try the TRSDOS27 interface ---

            mov     eax, pOSI

            mov     (COSI PTR [eax]).VPTR, offset TRSDOS27

            mcall   OSI_VTBL, Load, pOSI, pVDI

            or      eax, eax
            jz      Done

            mcall   OSI_VTBL, Unload, pOSI

    ; --- Try the RAPIDOS interface ---

            mov     eax, pOSI

            mov     (COSI PTR [eax]).VPTR, offset RAPIDOS

            mcall   OSI_VTBL, Load, pOSI, pVDI

            or      eax, eax
            jz      Done

            mcall   OSI_VTBL, Unload, pOSI

    ; --- Try the MULTIDOS interface ---

            mov     eax, pOSI

            mov     (COSI PTR [eax]).VPTR, offset MULTIDOS

            mcall   OSI_VTBL, Load, pOSI, pVDI

            or      eax, eax
            jz      Done

            mcall   OSI_VTBL, Unload, pOSI

    ; --- Must report an error ---

            mov     eax, ERROR_NOT_DOS_DISK

Done:       ret

GUI_LoadOSI         endp

;---------------------------------------------------------------------
; GUI_LoadVDI                              Load Virtual Disk Interface
;---------------------------------------------------------------------
; pVDI:         Pointer to VDI object
; hFile:        Handle to open diskette file
; Interface:    Selected VDI interface
;---------------------------------------------------------------------
; EAX:          Error code
;---------------------------------------------------------------------

GUI_LoadVDI         proc    pVDI:DWORD, hFile:HANDLE, Interface:DWORD

    ; --- Load requested interface ---

            cmp     Interface, -1
            je      Auto

            mov     edx, pVDI
            mov     eax, Interface
            mov     (CVDI PTR [edx]).VPTR, eax

            mcall   VDI_VTBL, Load, pVDI, hFile

            jmp     Done

    ; --- Try loading the DMK interface ---

Auto:       mov     eax, pVDI

            mov     (CVDI PTR [eax]).VPTR, offset DMK

            mcall   VDI_VTBL, Load, pVDI, hFile

            or      eax, eax
            jz      Done

            mcall   VDI_VTBL, Unload, pVDI

    ; --- Try the JV3 interface ---

            mov     eax, pVDI

            mov     (CVDI PTR [eax]).VPTR, offset JV3

            mcall   VDI_VTBL, Load, pVDI, hFile

            or      eax, eax
            jz      Done

            mcall   VDI_VTBL, Unload, pVDI

    ; --- Try the JV1 interface ---

            mov     eax, pVDI

            mov     (CVDI PTR [eax]).VPTR, offset JV1

            mcall   VDI_VTBL, Load, pVDI, hFile

            or      eax, eax
            jz      Done

            mcall   VDI_VTBL, Unload, pVDI

    ; --- Report an error ---

            mov     eax, ERROR_OPEN_FAILED

Done:       ret

GUI_LoadVDI         endp

;---------------------------------------------------------------------
; GUI_OpenFile                                Open a virtual disk file
;---------------------------------------------------------------------
; hWnd:     Window handle
; pFile:    Pointer to a null-terminated file name
; pVDI:     Pointer to Virtual Disk Interface VTBL
; pOSI:     Pointer to Operating System Interface VTBL
;---------------------------------------------------------------------
; EAX:      Error code
;---------------------------------------------------------------------

GUI_OpenFile        proc    hWnd:HWND, pFile:DWORD, pVDI:DWORD, pOSI:DWORD

            local   hFile:HANDLE

    ; --- Check whether a file is already open ---

            cmp     GUI_hFile, INVALID_HANDLE_VALUE
            je      @F

            fn      GUI_CloseFile, hWnd                                 ; Close it first
    
    ; --- Open the file ---

@@:         fn      CreateFile, pFile, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL

            cmp     eax, INVALID_HANDLE_VALUE
            jne     @F

            fn      GetLastError
            jmp     Done

@@:         mov     hFile, eax

    ; --- Load interfaces ---

            fn      GUI_LoadVDI, addr GUI_VDI, hFile, pVDI

            or      eax, eax
            jnz     CloseFile

            fn      GUI_LoadOSI, addr GUI_OSI, addr GUI_VDI, pOSI

            or      eax, eax
            jnz     UnloadVDI

    ; --- Retrieve the directory listing ---

            fn      GUI_List, hWnd

            or      eax, eax
            jnz     UnloadOSI

    ; --- Update the window title ---

            fn      SetWindowText, hWnd, pFile

            mov     eax, hFile
            mov     GUI_hFile, eax

            mov     eax, NO_ERROR
            jmp     Done

UnloadOSI:  push    eax
            mcall   OSI_VTBL, Unload, addr GUI_OSI
            pop     eax

UnloadVDI:  push    eax
            mcall   VDI_VTBL, Unload, addr GUI_VDI
            pop     eax

CloseFile:  push    eax
            fn      CloseHandle, hFile
            pop     eax
            
Done:       ret

GUI_OpenFile        endp

;---------------------------------------------------------------------
; GUI_CloseFile             Close the currently open virtual disk file
;---------------------------------------------------------------------
; hWnd:     Window handle
;---------------------------------------------------------------------
; No return value
;---------------------------------------------------------------------

GUI_CloseFile       proc    hWnd:HWND

            cmp     GUI_hFile, INVALID_HANDLE_VALUE
            je      Done

    ; --- Release interfaces ---
    
            mcall   OSI_VTBL, Unload, addr GUI_OSI

            mcall   VDI_VTBL, Unload, addr GUI_VDI

    ; --- Close file ---

            fn      CloseHandle, GUI_hFile

            mov     GUI_hFile, INVALID_HANDLE_VALUE

    ; --- Clear controls ---
    
            fn      GetDlgItem, hWnd, IDC_LIST

            fn      SendMessage, eax, LVM_DELETEALLITEMS, 0, 0

    ; --- Update the window title ---

            fn      SetWindowText, hWnd, "TRS-80 Virtual Floppy Disk Manager"

Done:       ret

GUI_CloseFile       endp

;---------------------------------------------------------------------
; GUI_OpenFileFromResDlg         Open file through interface selection
;---------------------------------------------------------------------
; hWnd:     Window handle
;---------------------------------------------------------------------
; No return
;---------------------------------------------------------------------

GUI_OpenFileFromResDlg  proc    hWnd:HWND

            local   pVDI:DWORD
            local   pOSI:DWORD
            local   Error:DWORD
            local   String[256]:BYTE

    ; --- Get the VDI selection and corresponding VTBL ---

            fn      GetDlgItem, hWnd, IDC_SEL_FF

            fn      SendMessage, eax, CB_GETCURSEL, 0, 0

            mov     edx, sizeof GUI_SEL_VDI
            mul     edx
            add     eax, offset GUI_SEL_VDI_Array

            mov     eax, (GUI_SEL_VDI PTR [eax]).pVTBL
            mov     pVDI, eax

    ; --- Get the OSI selection and corresponding VTBL ---

            fn      GetDlgItem, hWnd, IDC_SEL_OS

            fn      SendMessage, eax, CB_GETCURSEL, 0, 0

            mov     edx, sizeof GUI_SEL_OSI
            mul     edx
            add     eax, offset GUI_SEL_OSI_Array

            mov     eax, (GUI_SEL_OSI PTR [eax]).pVTBL
            mov     pOSI, eax

    ; --- Try to open the file ---

            fn      GetParent, hWnd

            mov     edx, eax

            fn      GUI_OpenFile, edx, offset GUI_FileName, pVDI, pOSI

            mov     Error, eax

    ; --- Show the result in the dialog ---

            fn      FormatMessage, FORMAT_MESSAGE_FROM_SYSTEM, NULL, Error, 0, addr String, sizeof String, NULL

            fn      GetDlgItem, hWnd, IDC_SEL_OUTPUT

            mov     edx, eax

            fn      SendMessage, edx, WM_SETTEXT, 0, addr String

            cmp     Error, NO_ERROR
            jne     Done

    ; --- Disable dropdown boxes and Open button ---

            fn      GetDlgItem, hWnd, IDC_SEL_FF

            fn      EnableWindow, eax, FALSE

            fn      GetDlgItem, hWnd, IDC_SEL_OS

            fn      EnableWindow, eax, FALSE

            fn      GetDlgItem, hWnd, IDC_SEL_OPEN

            fn      EnableWindow, eax, FALSE

    ; --- Change the Cancel button to Continue and give it focus ---

            fn      GetDlgItem, hWnd, IDC_SEL_CANCEL

            mov     edx, eax

            fn      SendMessage, edx, WM_SETTEXT, 0, "Continue"

Done:       ret
    
GUI_OpenFileFromResDlg  endp

;---------------------------------------------------------------------
; GUI_OpenFileFromSysDlg                    Open file passed by dialog
;---------------------------------------------------------------------
; hWnd:     Window handle
;---------------------------------------------------------------------
; No return
;---------------------------------------------------------------------

GUI_OpenFileFromSysDlg  proc    hWnd:HWND

            local   OFN:OPENFILENAME

    ; --- Clear OFN ---

            lea     edx, OFN
            mov     ecx, sizeof OFN
@@:         mov     byte ptr [edx + ecx - 1], 0
            loop    @B
            
    ; --- Init OFN ---
    
            mov     OFN.lStructSize, sizeof OFN

            mov     eax, hWnd
            mov     OFN.hwndOwner, eax

            fn      GetModuleHandle, NULL
            mov     OFN.hInstance, eax

            mov     OFN.lpstrFile, offset GUI_FileName
            mov     OFN.nMaxFile, sizeof GUI_FileName
            mov     OFN.lpstrFilter, offset GUI_Filter_1
            mov     OFN.Flags, OFN_FILEMUSTEXIST+OFN_HIDEREADONLY

    ; --- Present the dialog ---
    
            fn      GetOpenFileName, addr OFN

            or      eax, eax
            jz      Done

    ; --- Check whether auto-detect is enabled or not ---

            cmp     GUI_Config.AutoDetect, 0
            jne     @F

            fn      GetModuleHandle, NULL

            fn      DialogBoxParam, eax, IDD_OPEN, hWnd, offset GUI_Open, NULL

            jmp     Done

    ; --- Open file ---

@@:         fn      GUI_OpenFile, hWnd, offset GUI_FileName, -1, -1

            or      eax, eax
            jz      Done

            fn      ShowError, eax

Done:       ret
    
GUI_OpenFileFromSysDlg  endp

;---------------------------------------------------------------------
; GUI_OpenFileFromCmdLine             Open file passed by command line
;---------------------------------------------------------------------
; hWnd:     Window handle
;---------------------------------------------------------------------
; No return
;---------------------------------------------------------------------

GUI_OpenFileFromCmdLine proc    hWnd:HWND

    ; --- Check whether a file name was passed through the command-line ---

            fn      GetCommandLine

            fn      ToUpper, eax

            fn      ParseCmdLine, eax

            cmp     argc, 2
            jb      Done

    ; --- Copy the file name ---

            fn      szLen, argv[1 * sizeof DWORD]

            fn      MemCopy, argv[1 * sizeof DWORD], offset GUI_FileName, eax

    ; --- Check whether auto-detect is enabled or not ---

            cmp     GUI_Config.AutoDetect, 0
            jne     @F

            fn      GetModuleHandle, NULL

            fn      DialogBoxParam, eax, IDD_OPEN, hWnd, offset GUI_Open, NULL

            jmp     Done

    ; --- Open file ---

@@:         fn      GUI_OpenFile, hWnd, offset GUI_FileName, -1, -1

            or      eax, eax
            jz      Done

            fn      ShowError, eax

Done:       ret

GUI_OpenFileFromCmdLine endp

;---------------------------------------------------------------------
; GUI_LoadSettings                         Load settings from registry
;---------------------------------------------------------------------
; No input values
;---------------------------------------------------------------------
; EAX:      Error code
;---------------------------------------------------------------------

GUI_LoadSettings    proc

            local   hKey:PHKEY
            local   ValueType:DWORD
            local   Len:DWORD

            mov     Len, sizeof DWORD

            fn      RegOpenKeyEx, HKEY_CURRENT_USER, offset GUI_HKCU_Key, 0, KEY_QUERY_VALUE, addr hKey

            or      eax, eax
            jnz     Done

            mov     ecx, GUI_Settings_Count
            mov     edx, offset GUI_Settings

@@:         push    ecx
            push    edx
            
            fn      RegQueryValueEx, hKey, (GUI_SETTING PTR [edx]).pName, NULL, addr ValueType, (GUI_SETTING PTR [edx]).pValue, addr Len

            pop     edx
            pop     ecx

            add     edx, sizeof GUI_SETTING
            loop    @B

            fn      RegCloseKey, hKey

Done:       ret

GUI_LoadSettings    endp

;---------------------------------------------------------------------
; GUI_SaveSettings               Save current settings in the registry
;---------------------------------------------------------------------
; No input values
;---------------------------------------------------------------------
; EAX:      Error code
;---------------------------------------------------------------------

GUI_SaveSettings    proc

            local   hKey:PHKEY
            local   KeyStatus:DWORD
            local   Len:DWORD

            mov     Len, sizeof DWORD

            fn      RegCreateKeyEx, HKEY_CURRENT_USER, offset GUI_HKCU_Key, 0, "?", 0, KEY_WRITE, NULL, addr hKey, addr KeyStatus

            or      eax, eax
            jnz     Done

            mov     ecx, GUI_Settings_Count
            mov     edx, offset GUI_Settings

@@:         push    ecx
            push    edx
            
            fn      RegSetValueEx, hKey, (GUI_SETTING PTR [edx]).pName, 0, REG_DWORD, (GUI_SETTING PTR [edx]).pValue, sizeof DWORD

            pop     edx
            pop     ecx

            or      eax, eax
            jnz     Done
            
            add     edx, sizeof GUI_SETTING
            loop    @B

            fn      RegCloseKey, hKey

Done:       ret

GUI_SaveSettings    endp

end EntryPoint

