;---------------------------------------------------------------------
; JV3 Virtual Disk Interface                Copyright (C) Miguel Dutra
;---------------------------------------------------------------------

;---------------------------------------------------------------------
; JV3 Flags
;---------------------------------------------------------------------

JV3_Flag_Density        equ         10000000y                           ; Sector density (1:Double, 0:Single)
JV3_Flag_DAM            equ         01100000y                           ; Data Address Mark
JV3_Flag_Side           equ         00010000y                           ; Sector side (0:Side 0, 1:Side 1)
JV3_Flag_CRC            equ     	00001000y                           ; CRC status (0:Ok, 1:Error)
JV3_Flag_NonIBM         equ         00000100y                           ; 
JV3_Flag_Size           equ         00000011y                           ; Sector size (00:256, 11:512, 01:128, 10:1024)

;---------------------------------------------------------------------
; JV3 Data Address Marks (DAM) for Single Density
;---------------------------------------------------------------------

JV3_DAM_SD_0            equ         0FBh
JV3_DAM_SD_1            equ         0FAh
JV3_DAM_SD_2            equ         0F9h
JV3_DAM_SD_3            equ         0F8h

;---------------------------------------------------------------------
; JV3 Data Address Marks (DAM) for Double Density
;---------------------------------------------------------------------

JV3_DAM_DD_0            equ     	0FBh
JV3_DAM_DD_1            equ         0F8h
JV3_DAM_DD_2            equ         000h
JV3_DAM_DD_3            equ         000h

;---------------------------------------------------------------------
; JV3 Sector Sizes
;---------------------------------------------------------------------

JV3_Size_128            equ         01y
JV3_Size_256            equ         00y
JV3_Size_512            equ         11y
JV3_Size_1024           equ         10y

.data?

;---------------------------------------------------------------------
; JV3 Sector Header
;---------------------------------------------------------------------

JV3_SectorHeader    struct
    Track               BYTE        ?                                   ; Track number
    Sector              BYTE        ?                                   ; Sector number
    Flags               BYTE        ?                                   ; Sector flags
JV3_SectorHeader    ends

;---------------------------------------------------------------------
; JV3 Disk Header
;---------------------------------------------------------------------

JV3_Header  struct
    SectorHeader        JV3_SectorHeader    2901 dup (<>)
    WP                  BYTE                ?                           ; Write-Protected (00:No, FF:Yes)
JV3_Header  ends

;---------------------------------------------------------------------
; JV3 Class
;---------------------------------------------------------------------

CJV3    struct
                        CVDI        <>                                  ; Inherit all VDI class members
    Sectors             WORD        ?                                   ; Total number of sectors on disk
    pHeader             DWORD       ?                                   ; Pointer to disk header
    pXHeader            DWORD       ?                                   ; Pointer to extended header
CJV3    ends

.data

;---------------------------------------------------------------------
; JV3 Virtual Table
;---------------------------------------------------------------------

JV3_VTBL    struct                                                      ; Declare the structure and initializers
    Load                DWORD       JV3_Load                            ; VDI-derived functions
    Unload              DWORD       JV3_Unload
    Read                DWORD       JV3_Read
    Write               DWORD       JV3_Write
    GetTracks           DWORD       VDI_GetTracks
    GetSides            DWORD       VDI_GetSides
    GetSPT              DWORD       VDI_GetSPT
    GetDensity          DWORD       VDI_GetDensity
    GetFirstTrack       DWORD       VDI_GetFirstTrack
    GetLastTrack        DWORD       VDI_GetLastTrack
    GetFirstSide        DWORD       VDI_GetFirstSide
    GetLastSide         DWORD       VDI_GetLastSide
    GetFirstSector      DWORD       VDI_GetFirstSector
    GetLastSector       DWORD       VDI_GetLastSector
    GetDiskGeometry     DWORD       JV3_GetDiskGeometry                 ; JV3-specific functions
    Seek                DWORD       JV3_Seek
    FindSector          DWORD       JV3_FindSector
    GetSectorOffset     DWORD       JV3_GetSectorOffset
    GetSectorHeader     DWORD       JV3_GetSectorHeader
    GetSectorSize       DWORD       JV3_GetSectorSize
JV3_VTBL    ends

JV3                     JV3_VTBL    <>                                  ; Define a variable of that type

.code
    
;---------------------------------------------------------------------
; JV3_Load                                        Initialize interface
;---------------------------------------------------------------------
; pJV3:     Pointer to the calling JV3 object
; hFile:    Handle to an open Virtual Floppy Disk file
; Flags:    User-defined behavior modification flags
;---------------------------------------------------------------------
; EAX:      Error code
;---------------------------------------------------------------------

JV3_Load    proc    pJV3:DWORD, hFile:HANDLE, Flags:DWORD

            local   pHeader:DWORD
            local   pXHeader:DWORD
            local   Bytes:DWORD

            mov     pXHeader, 0                                         ; Initialize extended header pointer (used as flag)
            
   ; --- Load first header into memory ---

            fn      GlobalAlloc, GPTR, sizeof JV3_Header                ; Allocate memory for the disk header

            or      eax, eax
            jz      SysError

            mov     pHeader, eax

            fn      SetFilePointer, hFile, 0, NULL, FILE_BEGIN          ; Set file pointer at disk header

            cmp     eax, -1
            je      SysError

            fn      ReadFile, hFile, pHeader, sizeof JV3_Header, addr Bytes, NULL   ; Read it

            or      eax, eax
            jz      SysError

    ; --- Compute extended header offset ---

            mov     ecx, 2901                                           ; 2901 is the max number of sectors in the first JV3 block
            mov     edx, pHeader                                        ; EDX points to each 3-byte sector header
            mov     Bytes, sizeof JV3_Header                            ; "Bytes" holds the offset to the second JV3 block (including header)
            
@@:         mcall   JV3_VTBL, GetSectorSize, pJV3, edx

            add     Bytes, eax                                          ; Add this sector's size to the total sum
            add     edx, sizeof JV3_SectorHeader                        ; Advance to the next sector header
            loop    @B

    ; --- Load extended header ---

            fn      GetFileSize, hFile, NULL                            ; Check whether an extended header fits the file size

            cmp     eax, Bytes
            jb      @F

            fn      SetFilePointer, hFile, Bytes, NULL, FILE_BEGIN      ; Set file pointer at the extended header 

            cmp     eax, -1
            je      SysError

            fn      GlobalAlloc, GPTR, sizeof JV3_Header                ; Allocate memory for the extended header

            or      eax, eax
            jz      SysError

            mov     pXHeader, eax

            fn      ReadFile, hFile, pXHeader, sizeof JV3_Header, addr Bytes, NULL  ; Read it

            or      eax, eax
            jz      SysError

    ; --- Update member class variables ---

@@:         mov     edx, pJV3

            mov     eax, hFile                                          ; Update file handle
            mov     (CJV3 PTR [edx]).hFile, eax

            mov     eax, Flags                                          ; Update the behavior modification flags
            mov     (CJV3 PTR [edx]).Flags, eax
            
            mov     eax, pHeader                                        ; Update pointer to 1st header
            mov     (CJV3 PTR [edx]).pHeader, eax

            mov     eax, pXHeader                                       ; Update pointer to 2nd header
            mov     (CJV3 PTR [edx]).pXHeader, eax

    ; --- Find out and validate the disk geometry ---

            mcall   JV3_VTBL, GetDiskGeometry, pJV3                     ; Find out the disk properties
            
            mov     edx, pJV3

            cmp     (CJV3 PTR [edx]).Tracks, 35                         ; Tracks count must be between 35 and 96
            jb      IntError

            cmp     (CJV3 PTR [edx]).Tracks, 96
            ja      IntError

            cmp     (CJV3 PTR [edx]).SPT, 10                            ; Sectors Per Track must be between 10 and 30
            jb      IntError

            cmp     (CJV3 PTR [edx]).SPT, 30
            ja      IntError

            mov     eax, NO_ERROR
            jmp     Done
            
SysError:   fn      GetLastError
            jmp     Done

IntError:   mov     eax, ERROR_UNRECOGNIZED_MEDIA

Done:       ret

JV3_Load   endp

;---------------------------------------------------------------------
; JV3_Unload                                         Release interface
;---------------------------------------------------------------------
; pJV3:     Pointer to the calling JV3 object
;---------------------------------------------------------------------
; EAX:      Error code
;---------------------------------------------------------------------

JV3_Unload  proc    pJV3:DWORD

    ; --- Release memory allocated for 1st header ---
    
            mov     edx, pJV3

            fn      GlobalFree, (CJV3 PTR [edx]).pHeader

            or      eax, eax
            jz      @F

            fn      GetLastError
            jmp     Done

    ; --- Release memory allocated for extended header ---
    
@@:         mov     edx, pJV3

            cmp     (CJV3 PTR [edx]).pXHeader, 0
            je      Done
            
            fn      GlobalFree, (CJV3 PTR [edx]).pXHeader

            or      eax, eax
            jz      Done

            fn      GetLastError

Done:       ret
    
JV3_Unload  endp

;---------------------------------------------------------------------
; JV3_Read                                                 Read sector
;---------------------------------------------------------------------
; pJV3:     Pointer to the calling JV3 object
; pBuffer:  Pointer to sector data buffer
; Len:      Buffer length
; Track:    Track number
; Side:     Side number
; Sector:   Sector number
;---------------------------------------------------------------------
; EAX:      Error code
; ECX:      Sector size
; EDX:      Sector density (0:Single, 1:Double)
;---------------------------------------------------------------------

JV3_Read    proc    pJV3:DWORD, pBuffer:DWORD, Len:DWORD, Track:BYTE, Side:BYTE, Sector:BYTE

            local   SectorSize:DWORD
            local   Density:BYTE
            local   Bytes:DWORD
            
    ; --- Set file pointer ---

            mcall   JV3_VTBL, Seek, pJV3, Track, Side, Sector

            or      eax, eax
            jnz     Done
            
    ; --- Get sector properties ---

            mcall   JV3_VTBL, GetSectorHeader, pJV3, ecx                    ; Get pointer to sector header (ECX = Relative Sector)

            or      eax, eax
            jz      Error

            test    (JV3_SectorHeader PTR [eax]).Flags, JV3_Flag_Density    ; Get sector density
            setnz   Density
            
            mcall   JV3_VTBL, GetSectorSize, pJV3, eax                      ; Get sector size
            mov     SectorSize, eax
            
    ; --- Check buffer size ---

            cmp     Len, eax
            jbe     @F

            mov     Len, eax
            
    ; --- Read sector ---

@@:         mov     edx, pJV3
            
            fn      ReadFile, (CJV3 PTR [edx]).hFile, pBuffer, Len, addr Bytes, NULL

            or      eax, eax
            jnz     @F

            fn      GetLastError
            jmp     Done

    ; --- Set return registers ---
    
 @@:        mov     eax, NO_ERROR
            mov     ecx, SectorSize
            movzx   edx, Density
            jmp     Done
            
Error:      mov     eax, ERROR_SECTOR_NOT_FOUND

Done:       ret

JV3_Read    endp

;---------------------------------------------------------------------
; JV3_Write                                               Write sector
;---------------------------------------------------------------------
; pJV3:     Pointer to the calling JV3 object
; pBuffer:  Pointer to data buffer
; Len:      Buffer length
; Track:    Track number
; Side:     Side number
; Sector:   Sector number
;---------------------------------------------------------------------
; EAX:      Error code
;---------------------------------------------------------------------

JV3_Write   proc    pJV3:DWORD, pBuffer:DWORD, Len:DWORD, Track:BYTE, Side:BYTE, Sector:BYTE

            local   Bytes:DWORD
            
    ; --- Set file pointer ---

            mcall   JV3_VTBL, Seek, pJV3, Track, Side, Sector

            or      eax, eax
            jnz     Done
            
    ; --- Check data length ---

            mcall   JV3_VTBL, GetSectorHeader, pJV3, ecx                    ; Get pointer to sector header (ECX = Relative Sector)

            or      eax, eax
            jz      Error

            mcall   JV3_VTBL, GetSectorSize, pJV3, eax                      ; Get sector size

            cmp     Len, eax
            jbe     @F

            mov     Len, eax
            
    ; --- Write sector ---

@@:         mov     edx, pJV3
            
            fn      WriteFile, (CJV3 PTR [edx]).hFile, pBuffer, Len, addr Bytes, NULL

            or      eax, eax
            jnz     @F

            fn      GetLastError
            jmp     Done

 @@:        mov     eax, NO_ERROR
            jmp     Done
            
Error:      mov     eax, ERROR_SECTOR_NOT_FOUND

Done:       ret

JV3_Write   endp

;---------------------------------------------------------------------
; JV3_GetDiskGeometry                     Find out the disk properties
;---------------------------------------------------------------------
; pJV3:     Pointer to the calling JV3 object
;---------------------------------------------------------------------
; No return (member variables updated directly)
;---------------------------------------------------------------------

JV3_GetDiskGeometry proc    pJV3:DWORD

            local   Sectors:WORD
            local   LowestTrack:BYTE
            local   HighestTrack:BYTE
            local   LowestSector:BYTE
            local   HighestSector:BYTE
            local   SPT:BYTE
            local   Sides:BYTE
            local   Density:BYTE
            local   ScanFlag:BYTE

    ; --- Initialize local variables ---

            mov     Sectors, 0
            mov     LowestTrack, -1
            mov     HighestTrack, 0
            mov     LowestSector, -1
            mov     HighestSector, 0
            mov     SPT, 0
            mov     Sides, 1
            mov     Density, -1
            mov     ScanFlag, 0
            
    ; --- Scan all header entries for the lowest and highest track ---

            mov     edx, pJV3
            mov     edx, (CJV3 PTR [edx]).pHeader

Restart:    mov     ecx, 2901

    ; --- Detect lowest and highest track numbers ---

Scan:       mov     al, (JV3_SectorHeader PTR [edx]).Track              ; Get sector's Track number

            cmp     al, -1                                              ; Stop when an entry with 0xFFs is found
            je      Update

            cmp     al, LowestTrack                                     ; Is the track number lower than the stored one?
            jae     @F

            mov     LowestTrack, al

@@:         cmp     al, HighestTrack                                    ; Is the track number higher than the stored one?
            jbe     @F

            mov     HighestTrack, al

@@:         cmp     al, 2                                               ; Execute the remaining tests only for track 2
            jne     Next

    ; --- Detect number of sides ---
    
            test    (JV3_SectorHeader PTR [edx]).Flags, JV3_Flag_Side   ; Get sector side
            jz      @F

            mov     Sides, 2

    ; --- Detect lowest and highest sector numbers ---

@@:         mov     al, (JV3_SectorHeader PTR [edx]).Sector             ; Get sector number

            cmp     al, -1                                              ; Shouldn't find an FF here, but...
            je      Update

            cmp     al, LowestSector                                    ; Is the sector number lower than the stored one?
            jae     @F

            mov     LowestSector, al

@@:         cmp     al, HighestSector                                   ; Is the sector number higher than the stored one?
            jbe     @F

            mov     HighestSector, al

    ; --- Compare density ---
    
@@:         test    (JV3_SectorHeader PTR [edx]).Flags, JV3_Flag_Density    ; Get sector density
            setnz   al

            cmp     Density, -1                                         ; Is density already init'd?
            jne     @F

            mov     Density, al                                         ; Assume same density for the disk

@@:         cmp     Density, al                                         ; Is this a mixed density disk?
            je      @F

            mov     Density, 2                                          ; Set mixed density value

    ; --- Count side 0 sectors ---

;@@:         cmp     (JV3_SectorHeader PTR [edx]).Track, 2               ; Track 2?
;            jne     @F

@@:         test    (JV3_SectorHeader PTR [edx]).Flags, JV3_Flag_Side   ; Is this sector on side 0?
            jnz     Next

            inc     SPT                                                 ; Add to the total
    
    ; --- Advance to the next sector and loop ---
    
Next:       add     edx, sizeof JV3_SectorHeader
            inc     Sectors
            loop    Scan

    ; --- Repeat the process for the extended header ---

            cmp     ScanFlag, 0                                         ; Have we processed both headers already?
            jne     Update

            inc     ScanFlag                                            ; Flag it as being processed now

            mov     edx, pJV3                                           ; Point to the extended header
            mov     edx, (CJV3 PTR [edx]).pXHeader

            or      edx, edx                                            ; Check if the extended header ptr is not null and go back
            jnz     Restart

    ; --- Update member variables ---

Update:     mov     edx, pJV3

            mov     al, HighestTrack                                    ; Update number of disk tracks
            sub     al, LowestTrack
            inc     al
            mov     (CJV3 PTR [edx]).Tracks, al

            mov     al, LowestTrack                                     ; Update number of first track on disk
            mov     (CJV3 PTR [edx]).FirstTrack, al

            mov     al, HighestTrack                                    ; Update number of last track on disk
            mov     (CJV3 PTR [edx]).LastTrack, al

            mov     al, Sides                                           ; Update number of disk sides
            mov     (CJV3 PTR [edx]).Sides, al

            mov     al, SPT                                             ; Update number of sectors per track
            mov     (CJV3 PTR [edx]).SPT, al

            mov     al, LowestSector                                    ; Update number of first sector on track
            mov     (CJV3 PTR [edx]).FirstSector, al

            mov     al, HighestSector                                   ; Update number of last track on track
            mov     (CJV3 PTR [edx]).LastSector, al

            mov     al, Density                                         ; Update the disk density
            mov     (CJV3 PTR [edx]).Density, al

            mov     ax, Sectors                                         ; Update total number of disk sectors
            mov     (CJV3 PTR [edx]).Sectors, ax

            ret

JV3_GetDiskGeometry endp

;---------------------------------------------------------------------
; JV3_Seek                      Set file pointer to a requested sector
;---------------------------------------------------------------------
; pJV3:     Pointer to the calling JV3 object
; Track:    Track number
; Side:     Side number
; Sector:   Sector number
;---------------------------------------------------------------------
; EAX:      Error code
; ECX:      Relative sector
;---------------------------------------------------------------------

JV3_Seek    proc    pJV3:DWORD, Track:BYTE, Side:BYTE, Sector:BYTE

            local   RelSector:WORD

    ; --- Validate the arguments ---

            mov     edx, pJV3

            mov     al, Track                                           
            cmp     al, (CJV3 PTR [edx]).LastTrack                 ; Track must be equal or lower than LastTrack
            ja      Error

            mov     al, Side
            cmp     al, (CJV3 PTR [edx]).Sides                     ; Side must be lower than the actual number of Sides (1 or 2)
            jnb     Error
            
            mov     al, Sector
            cmp     al, (CJV3 PTR [edx]).LastSector                ; Sector must be equal or lower than LastSector
            ja      Error

    ; --- Locate the sector ---

            mcall   JV3_VTBL, FindSector, pJV3, Track, Side, Sector

            cmp     eax, -1
            je      Error

            mov     RelSector, ax

    ; --- Compute sector offset ---

            mcall   JV3_VTBL, GetSectorOffset, pJV3, ax

            or      eax, eax
            jz      Error

    ; --- Set file pointer ---

            mov     edx, pJV3
            
            fn      SetFilePointer, (CJV3 PTR [edx]).hFile, eax, NULL, FILE_BEGIN

            cmp     eax, -1
            jne     @F

            fn      GetLastError
            jmp     Done

@@:         mov     eax, NO_ERROR
            movzx   ecx, RelSector
            jmp     Done
            
Error:      mov     eax, ERROR_SECTOR_NOT_FOUND

Done:       ret

JV3_Seek    endp

;---------------------------------------------------------------------
; JV3_FindSector                            Find a sector by its C/H/S
;---------------------------------------------------------------------
; pJV3:     Pointer to the calling JV3 object
; Track:    Track number
; Side:     Side number
; Sector:   Sector number
;---------------------------------------------------------------------
; EAX:      Relative sector number or -1 if not found
;---------------------------------------------------------------------

JV3_FindSector  proc    pJV3:DWORD, Track:BYTE, Side:BYTE, Sector:BYTE

            local   RelSector:WORD

            mov     RelSector, 0
            
            mov     edx, pJV3
            movzx   ecx, (CJV3 PTR [edx]).Sectors
            mov     eax, (CJV3 PTR [edx]).pHeader

Check:      mov     dl, (JV3_SectorHeader PTR [eax]).Track              ; Check if the sector's track matches the requested track
            cmp     dl, Track
            jne     Next

            mov     dl, (JV3_SectorHeader PTR [eax]).Sector             ; Check if the sector's sector matches the requested sector
            cmp     dl, Sector
            jne     Next

            test    (JV3_SectorHeader PTR [eax]).Flags, JV3_Flag_Side   ; Check if the sector's side matches the requested side
            setnz   dl
            cmp     dl, Side
            je      Found

Next:       add     eax, sizeof JV3_SectorHeader                        ; Advance to the next entry

            inc     RelSector
            cmp     RelSector, 2901                                     ; Check whether we crossed the first header limit
            jne     @F

            mov     edx, pJV3                                           ; Redirect pointer to the extended header
            mov     eax, (CJV3 PTR [edx]).pXHeader
            
@@:         loop    Check                                               ; Go back

Error:      mov     eax, -1                                             ; If no matches found at all, return -1
            jmp     Done

Found:      movzx   eax, RelSector                                      ; Return the current index in EAX

Done:       ret

JV3_FindSector  endp

;---------------------------------------------------------------------
; JV3_GetSectorOffset                        Compute a sector's offset
;---------------------------------------------------------------------
; pJV3:     Pointer to the calling JV3 object
; Sector:   Relative sector number
;---------------------------------------------------------------------
; EAX:      File offset to the requested sector or NULL on error
;---------------------------------------------------------------------

JV3_GetSectorOffset proc    pJV3:DWORD, Sector:WORD

            mov     edx, sizeof JV3_Header                              ; EDX holds the offset accumulator
            
            xor     cx, cx                                              ; CX holds the sector counter

@@:         cmp     cx, Sector                                          ; Current sector matches the requested one?
            je      Done

            mcall   JV3_VTBL, GetSectorHeader, pJV3, cx                 ; Get pointer to sector header

            or      eax, eax
            jz      Error

            cmp     (JV3_SectorHeader PTR [eax]).Sector, 0FFh           ; Check whether we came across a free sector
            je      Error
            
            mcall   JV3_VTBL, GetSectorSize, pJV3, eax

            add     edx, eax                                            ; Add its size to the offset accumulator
            
            inc     cx                                                  ; Move to the next sector
            jmp     @B

Error:      mov     edx, NULL                                           ; On error return null

Done:       mov     eax, edx                                            ; Return the result in EAX

            ret

JV3_GetSectorOffset endp

;---------------------------------------------------------------------
; JV3_GetSectorHeader                  Return pointer to sector header
;---------------------------------------------------------------------
; pJV3:     Pointer to the calling JV3 object
; Sector:   Relative sector number
;---------------------------------------------------------------------
; EAX:      Pointer to sector header or NULL on error
;---------------------------------------------------------------------

JV3_GetSectorHeader proc    pJV3:DWORD, Sector:WORD

            push    ecx
            push    edx

            movzx   eax, Sector                                         ; Initialize the variables
            mov     ecx, sizeof JV3_SectorHeader

            mov     edx, pJV3
            cmp     ax, (CJV3 PTR [edx]).Sectors                        ; Check if the requested sector is valid
            jnb     Error

            cmp     ax, 2901                                            ; Check whether the requested sector is in the first or second block
            jnb     Extended

            mul     cx                                                  ; AX = (Sector * Sector_Header_Size) + 1st_Header_Starting_Address
            mov     edx, pJV3
            add     eax, (CJV3 PTR [edx]).pHeader
            jmp     Done
            
Extended:   sub     ax, 2901                                            ; AX = ((Sector - 2901) * Sector_Header_Size) + 2nd_Header_Starting_Address
            mul     cx
            mov     edx, pJV3
            add     eax, (CJV3 PTR [edx]).pXHeader

            cmp     (CJV3 PTR [edx]).pXHeader, 0                        ; Just be sure that the extended header exists
            jne     Done

Error:      mov     eax, NULL                                           ; If not, return null

Done:       pop     edx
            pop     ecx

            ret

JV3_GetSectorHeader endp

;---------------------------------------------------------------------
; JV3_GetSectorSize                              Determine sector size
;---------------------------------------------------------------------
; pJV3:     Pointer to the calling JV3 object
; pJV3HDR:  Pointer to the sector header
;---------------------------------------------------------------------
; EAX:      Sector size (128, 256, 512 or 1024)
;---------------------------------------------------------------------

JV3_GetSectorSize   proc    pJV3:DWORD, pJV3HDR:DWORD

            mov     eax, pJV3HDR

            mov     al, (JV3_SectorHeader PTR [eax]).Flags
            
            mov     ah, al                                              ; Check whether the sector is free or occupied
            and     ah, 0FCh
            cmp     ah, 0FCh
            jne     @F
            
            not     al                                                  ; Bits must be inverted if sector is free
            
@@:         and     al, JV3_Flag_Size                                   ; Isolate size bits

            cmp     al, JV3_Size_256                                    ; Case 256 bytes
            jne     @F

            mov     eax, 256
            jmp     Done
            
@@:         cmp     al, JV3_Size_128                                    ; Case 128 bytes
            jne     @F

            mov     eax, 128
            jmp     Done

@@:         cmp     al, JV3_Size_512                                    ; Case 512 bytes
            jne     @F

            mov     eax, 512
            jmp     Done

@@:         mov     eax, 1024                                           ; Otherwise, 1024 bytes

Done:       ret
            
JV3_GetSectorSize   endp

