;---------------------------------------------------------------------
; Virtual Disk Interface                    Copyright (C) Miguel Dutra
;---------------------------------------------------------------------

.data?

;---------------------------------------------------------------------
; VDI Class
;---------------------------------------------------------------------

CVDI   struct
    VPTR                DWORD       ?                                   ; Pointer to the object's VTBL
    hFile               HANDLE      ?                                   ; Handle to the disk image file
    Flags               DWORD       ?                                   ; Behavior modification switches
    Tracks              BYTE        ?                                   ; Number of disk tracks
    Sides               BYTE        ?                                   ; Number of disk sides
    SPT                 BYTE        ?                                   ; Number of sectors per track
    Density             BYTE        ?                                   ; Disk density (0:Single, 1:Double, 2:Mixed)
    FirstTrack          BYTE        ?                                   ; Number of first track on disk
    LastTrack           BYTE        ?                                   ; Number of last track on disk
    FirstSide           BYTE        ?                                   ; Number of first disk side
    LastSide            BYTE        ?                                   ; Number of last disk side
    FirstSector         BYTE        ?                                   ; Number of first sector on track
    LastSector          BYTE        ?                                   ; Number of last sector on track
CVDI   ends

.data

;---------------------------------------------------------------------
; VDI Virtual Table
;---------------------------------------------------------------------

VDI_VTBL    struct                                                      ; Declare the structure and initializers
    Load                DWORD       VDI_Load
    Unload              DWORD       VDI_Unload
    Read                DWORD       VDI_Read
    Write               DWORD       VDI_Write
    GetTracks           DWORD       VDI_GetTracks
    GetSides            DWORD       VDI_GetSides
    GetSPT              DWORD       VDI_GetSPT
    GetDensity          DWORD       VDI_GetDensity
    GetFirstTrack       DWORD       VDI_GetFirstTrack
    GetLastTrack        DWORD       VDI_GetLastTrack
    GetFirstSide        DWORD       VDI_GetFirstSide
    GetLastSide         DWORD       VDI_GetLastSide
    GetFirstSector      DWORD       VDI_GetFirstSector
    GetLastSector       DWORD       VDI_GetLastSector
VDI_VTBL    ends

VDI                     VDI_VTBL    <>                                  ; Define a variable of that type

.code
    
;---------------------------------------------------------------------
; VDI_Load                                        Initialize interface
;---------------------------------------------------------------------
; pVDI:     Pointer to the calling VDI object
; hFile:    Handle to an open Virtual Floppy Disk file
; Flags:    User-defined behavior modification flags
;---------------------------------------------------------------------
; EAX:      Error code
;---------------------------------------------------------------------

VDI_Load    proc    pVDI:DWORD, hFile:HANDLE, Flags:DWORD

            mov     eax, ERROR_CALL_NOT_IMPLEMENTED
            ret

VDI_Load    endp

;---------------------------------------------------------------------
; VDI_Unload                                         Release interface
;---------------------------------------------------------------------
; pVDI:     Pointer to the calling VDI object
;---------------------------------------------------------------------
; EAX:      Error code
;---------------------------------------------------------------------

VDI_Unload  proc    pVDI:DWORD

            mov     eax, ERROR_CALL_NOT_IMPLEMENTED
            ret

VDI_Unload  endp

;---------------------------------------------------------------------
; VDI_Read                                                 Read sector
;---------------------------------------------------------------------
; pVDI:     Pointer to the calling VDI object
; pBuffer:  Pointer to data buffer
; Len:      Buffer length
; Track:    Track number
; Side:     Side number
; Sector:   Sector number
;---------------------------------------------------------------------
; EAX:      Error code
; ECX:      Sector size
; EDX:      Sector density (0:Single, 1:Double)
;---------------------------------------------------------------------

VDI_Read    proc    pVDI:DWORD, pBuffer:DWORD, Len:DWORD, Track:BYTE, Side:BYTE, Sector:BYTE

            mov     eax, ERROR_CALL_NOT_IMPLEMENTED
            ret

VDI_Read    endp

;---------------------------------------------------------------------
; VDI_Write                                               Write sector
;---------------------------------------------------------------------
; pVDI:     Pointer to the calling VDI object
; pBuffer:  Pointer to data buffer
; Len:      Buffer length
; Track:    Track number
; Side:     Side number
; Sector:   Sector number
;---------------------------------------------------------------------
; EAX:      Error code
;---------------------------------------------------------------------

VDI_Write   proc    pVDI:DWORD, pBuffer:DWORD, Len:DWORD, Track:BYTE, Side:BYTE, Sector:BYTE

            mov     eax, ERROR_CALL_NOT_IMPLEMENTED
            ret

VDI_Write   endp

;---------------------------------------------------------------------
; VDI_GetTracks                    Return the number of tracks on disk
;---------------------------------------------------------------------
; pVDI:     Pointer to the calling VDI object
;---------------------------------------------------------------------
; EAX:      Number of tracks on disk
;---------------------------------------------------------------------

VDI_GetTracks   proc    pVDI:DWORD

            mov     eax, pVDI
            movzx   eax, (CVDI PTR [eax]).Tracks
            ret

VDI_GetTracks   endp

;---------------------------------------------------------------------
; VDI_GetSides                         Return the number of disk sides
;---------------------------------------------------------------------
; pVDI:     Pointer to the calling VDI object
;---------------------------------------------------------------------
; EAX:      Number of disk sides
;---------------------------------------------------------------------

VDI_GetSides    proc    pVDI:DWORD

            mov     eax, pVDI
            movzx   eax, (CVDI PTR [eax]).Sides
            ret

VDI_GetSides    endp

;---------------------------------------------------------------------
; VDI_GetSPT                    Return the number of sectors per track
;---------------------------------------------------------------------
; pVDI:     Pointer to the calling VDI object
;---------------------------------------------------------------------
; EAX:      Number of tracks on disk
;---------------------------------------------------------------------

VDI_GetSPT  proc    pVDI:DWORD

            mov     eax, pVDI
            movzx   eax, (CVDI PTR [eax]).SPT
            ret

VDI_GetSPT  endp

;---------------------------------------------------------------------
; VDI_GetDensity                               Return the disk density
;---------------------------------------------------------------------
; pVDI:     Pointer to the calling VDI object
;---------------------------------------------------------------------
; EAX:      Disk density (0:Single, 1:Double)
;---------------------------------------------------------------------

VDI_GetDensity  proc    pVDI:DWORD

            mov     eax, pVDI
            movzx   eax, (CVDI PTR [eax]).Density
            ret

VDI_GetDensity  endp

;---------------------------------------------------------------------
; VDI_GetFirstTrack                     Return the first track on disk
;---------------------------------------------------------------------
; pVDI:     Pointer to the calling VDI object
;---------------------------------------------------------------------
; EAX:      Number of the first track on disk
;---------------------------------------------------------------------

VDI_GetFirstTrack   proc    pVDI:DWORD

            mov     eax, pVDI
            movzx   eax, (CVDI PTR [eax]).FirstTrack
            ret

VDI_GetFirstTrack   endp

;---------------------------------------------------------------------
; VDI_GetLastTrack                       Return the last track on disk
;---------------------------------------------------------------------
; pVDI:     Pointer to the calling VDI object
;---------------------------------------------------------------------
; EAX:      Number of the last track on disk
;---------------------------------------------------------------------

VDI_GetLastTrack    proc    pVDI:DWORD

            mov     eax, pVDI
            movzx   eax, (CVDI PTR [eax]).LastTrack
            ret

VDI_GetLastTrack    endp

;---------------------------------------------------------------------
; VDI_GetFirstSide                          Return the first disk side
;---------------------------------------------------------------------
; pVDI:     Pointer to the calling VDI object
;---------------------------------------------------------------------
; EAX:      Number of first disk side
;---------------------------------------------------------------------

VDI_GetFirstSide    proc    pVDI:DWORD

            mov     eax, pVDI
            movzx   eax, (CVDI PTR [eax]).FirstSide
            ret

VDI_GetFirstSide    endp

;---------------------------------------------------------------------
; VDI_GetLastSide                            Return the last disk side
;---------------------------------------------------------------------
; pVDI:     Pointer to the calling VDI object
;---------------------------------------------------------------------
; EAX:      Number of tracks on disk
;---------------------------------------------------------------------

VDI_GetLastSide proc    pVDI:DWORD

            mov     eax, pVDI
            movzx   eax, (CVDI PTR [eax]).LastSide
            ret

VDI_GetLastSide endp

;---------------------------------------------------------------------
; VDI_GetFirstSector                  Return the first sector on track
;---------------------------------------------------------------------
; pVDI:     Pointer to the calling VDI object
;---------------------------------------------------------------------
; EAX:      Number of first sector on track
;---------------------------------------------------------------------

VDI_GetFirstSector  proc    pVDI:DWORD

            mov     eax, pVDI
            movzx   eax, (CVDI PTR [eax]).FirstSector
            ret

VDI_GetFirstSector  endp

;---------------------------------------------------------------------
; VDI_GetLastSector                    Return the last sector on track
;---------------------------------------------------------------------
; pVDI:     Pointer to the calling VDI object
;---------------------------------------------------------------------
; EAX:      Number of last sector on track
;---------------------------------------------------------------------

VDI_GetLastSector   proc    pVDI:DWORD

            mov     eax, pVDI
            movzx   eax, (CVDI PTR [eax]).LastSector
            ret

VDI_GetLastSector   endp

